--------------------------------------------------------------------------------
-- Title   : Top - MIK pulse extension function
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Top sheet of MIK pulse extension block
--    inTTL(0) is duplicated to outTTL[4 donwto 1]
--    outputs are 4 s width pulses
--    outTTL(0) is not used as the board IO connector is hard configured as
--    input
--------------------------------------------------------------------------------
-- File     : MIK_top.vhdl
-- Revision : x.x.x
-- Created  : April 28, 2015
-- Updated  : April 30, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2015 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the Lesser GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity MIK_top is
  port (
    p_MIK_inTTL             : in  std_logic;                     -- input TTL
    p_MIK_outTTL            : out std_logic_vector (4 downto 0); -- TTL outputs
    p_MIK_outPECL           : out std_logic_vector (4 downto 0); -- LVPECL outputs
    p_MIK_clk1kHz           : in  std_logic;
    p_MIK_clk500mHz         : in  std_logic;
    p_MIK_clk60MHz          : in  std_logic;
    p_MIK_clk240MHz         : in  std_logic;
    p_MIK_pulseWidth        : in  std_logic_vector (31 downto 0);
    p_MIK_missingPulseDelay : in  std_logic_vector (31 downto 0);
    p_MIK_csw               : in  std_logic_vector (1 downto 0);
    p_MIK_reset             : in  std_logic;
    p_MIK_pulseMon          : out std_logic_vector (31 downto 0); -- pulses monitor
    p_MIK_led               : out std_logic_vector (1 downto 0)
  );
end entity MIK_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_MIK_top of MIK_top is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_outPulse    : std_logic;

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

    -- Input pulse detection block
    pulseDetection : entity work.MIK_detection (rtl_MIK_detection)
      port map (
        p_reset      => p_MIK_reset,
        p_clk60MHz   => p_MIK_clk60MHz,
        p_clk240MHz  => p_MIK_clk240MHz,
        p_pulseIn    => p_MIK_inTTL, -- TTL in
        p_pulseWidth => p_MIK_pulseWidth,
        p_csw        => p_MIK_csw(1),
        p_pulseOut   => s_outPulse   -- out
      );

    -- Signal duplication block
    duplication : entity work.MIK_duplication (rtl_MIK_duplication)
      port map (
        p_in      => s_outPulse,   -- in
        p_outTTL  => p_MIK_outTTL, -- TTL out
        p_outPECL => p_MIK_outPECL -- LVPECL out ; not used
      );

    -- Signal monitoring block
    -- The time betwwen 2 input pulses in stored in MONITOR register (in ms)
    pulseMonitor : entity work.MIK_monitoring (rtl_MIK_monitoring)
      port map (
        p_inTTL   => p_MIK_inTTL,    -- TTL input
        p_clk1kHz => p_MIK_clk1kHz,
        p_reset   => p_MIK_reset,
        p_monitor => p_MIK_pulseMon  -- register with the time value between two pulses
      );

    -- LEDs management for the duplication block
    -- The green LED is blinked for 5s after a reset or after a input pulse is
    -- dtected. It remains ON in other cases.
    -- The red LED is off after a reset. It gos ON if no input signal is detected
    -- after delay defined in the missingPulseDelay register.
    leds : entity work.MIK_leds (rtl_MIK_leds)
      port map (
        p_inTTL             => p_MIK_inTTL,  -- TTL input
        p_clk1kHz           => p_MIK_clk1kHz,
        p_clk500mHz         => p_MIK_clk500mHz,
        p_clk60MHz          => p_MIK_clk60MHz,
        p_missingPulseDelay => p_MIK_missingPulseDelay,
        p_csw               => p_MIK_csw(0),
        p_reset             => p_MIK_reset,
        p_led               => p_MIK_led
      );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

end architecture rtl_MIK_top;
