--------------------------------------------------------------------------------
-- Title   : clock padding
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : clock padding
--------------------------------------------------------------------------------
-- File     : clkpadding_top.vhdl
-- Revision : x.x.x
-- Created  : July 19, 2014
-- Updated  : October 02, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2018 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the Lesser GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clkpadding_padding is
  generic (
    g_gap_detect     : unsigned (15 downto 0);
    g_padding_period : unsigned (15 downto 0);
    g_pulse_high     : unsigned (15 downto 0)
  );
  port (
    p_clkPADDING_clk500mHz : in  std_logic;
    p_clkPADDING_clk60MHz  : in  std_logic;
    p_clkPADDING_clk_mon   : in  std_logic; -- monitored clock
    p_clkPADDING_reset     : in  std_logic; -- global reset
    p_clkPADDING_outTTL    : out std_logic; -- TTL output ; output clock
    p_clkPADDING_LED       : out std_logic
  );
end entity clkpadding_padding;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clkpadding_padding of clkpadding_padding is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_cnt1 : unsigned (15 downto 0); -- counter
  signal s_cnt2 : unsigned (15 downto 0); -- counter

  signal s_start         : std_logic;
  signal s_underProgress : std_logic;
  signal s_clkPadding    : std_logic;
  signal s_clk_OUT       : std_logic;
  signal s_reset         : std_logic;
  signal s_extCLKup      : std_logic;
  signal s_led           : std_logic;
  signal s_ledHold       : std_logic;

 ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Detection of a gap in the input clock
  process (p_clkPADDING_reset, s_reset, p_clkPADDING_clk60MHz)
  begin
   if ((p_clkPADDING_reset = '1') or (s_reset = '1')) then
     s_cnt1  <= (OTHERS => '0');
     s_start <= '0';
   elsif (rising_edge(p_clkPADDING_clk60MHz)) then
     if (s_cnt1 = g_gap_detect) then
       s_start <= '1';
       s_cnt1  <= s_cnt1;
     else
       s_cnt1  <= s_cnt1 + 1;
     end if;
   end if;
  end process;

  -- Generation of a padding clock if a gap is detected
  -- in the input clock
  process (p_clkPADDING_reset, p_clkPADDING_clk60MHz, p_clkPADDING_clk_mon)
  begin
    if (p_clkPADDING_reset = '1') then
      s_cnt2          <= (OTHERS => '0');
      s_clkPadding    <= '0';
    elsif (rising_edge(p_clkPADDING_clk60MHz)) then
      if (s_start = '1') then
        s_cnt2  <= s_cnt2 + 1;
        if (s_cnt2 < g_pulse_high) then
          s_clkPadding <= '1';
        elsif (s_cnt2 < g_padding_period) then
          s_clkPadding <= '0';
        else
          s_cnt2       <= (OTHERS => '0');
          s_clkPadding <= '0';
        end if;
      else
        s_cnt2          <= (OTHERS => '0');
        s_clkPadding    <= '0';
      end if;
    end if;
  end process;

  -- Reset the padding if the external clock is on
  process (p_clkPADDING_reset, s_reset, p_clkPADDING_clk_mon)
  begin
    if ((p_clkPADDING_reset = '1') or (s_reset = '1')) then
      s_extCLKup <= '0';
    elsif (rising_edge(p_clkPADDING_clk_mon)) then
      s_extCLKup <= '1';
    end if;
  end process;

  process (p_clkPADDING_reset, p_clkPADDING_clk60MHz)
  begin
    if (p_clkPADDING_reset = '1') then
      s_reset <= '0';
    elsif (rising_edge(p_clkPADDING_clk60MHz)) then
      if (s_extCLKup = '1') then
        s_reset <= '1';
      else
        s_reset <= '0';
      end if;
    end if;
  end process;

  -- Multiplex the output according to the input state
  with s_start select
    s_clk_OUT <=
      s_clkPadding         when '1',
      p_clkPADDING_clk_mon when OTHERS;

  -- Trigger red LED
  process (p_clkPADDING_reset, s_start, p_clkPADDING_clk500mHz)
  begin
    if ((p_clkPADDING_reset = '1') or (s_start = '1')) then
      s_led     <= '1';
      s_ledHold <= '1';
    elsif (rising_edge(p_clkPADDING_clk500mHz)) then
      s_led     <= s_ledHold;
      s_ledHold <= '0';
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  p_clkPADDING_outTTL <= s_clk_OUT;
  p_clkPADDING_LED    <= s_led; -- red LED

end architecture rtl_clkpadding_padding;
