-------------------------------------------------------------------------------
-- Title   : Top-up gating and beam lost management
-- Project : TimEX3
-------------------------------------------------------------------------------
-- Description : Set a bit during 2s after a top-up injection window or a beam
--               lost window. The bit will be set for window width duration + 2s
--               those it can be read by polling through soft
-------------------------------------------------------------------------------
-- File     : topup_stateBit.vhdl
-- Revision : x.x.x
-- Created  : November 06, 2013
-- Updated  : November 06, 2013
-------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
-------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
-------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------
-- Modifications :
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-------------------------------------------------------------------------------
--------------------------------- ENTITY --------------------------------------
-------------------------------------------------------------------------------
entity topup_stateBit is
  port (
    p_clk500mHz        : in  std_logic;
    p_gateWindow       : in  std_logic;
    p_beamLost         : in  std_logic;
    p_stateBitGate     : out std_logic;
    p_stateBitBeamLost : out std_logic;
    p_reset            : in  std_logic
  );
end entity topup_stateBit;

-------------------------------------------------------------------------------
------------------------------- ARCHITECTURE ----------------------------------
-------------------------------------------------------------------------------
architecture rtl_topup_stateBit of topup_stateBit is
  -----------------------------------------------------------------------------
  -- constant
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- signal
  -----------------------------------------------------------------------------
  signal s_stateBitGate     : std_logic;
  signal s_gateEnd          : std_logic;
  signal s_stateBitbeamLost : std_logic;
  signal s_beamLostEnd      : std_logic;
  signal s_cntGateWidth     : unsigned (1 downto 0);
  signal s_cntbeamLostWidth : unsigned (1 downto 0);

  -----------------------------------------------------------------------------
  -- Components
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  --------------------------------- Main --------------------------------------
  -----------------------------------------------------------------------------
  begin

   -- Start
  process (p_reset, s_gateEnd, p_gateWindow)
  begin
    if ((p_reset = '1') or (s_gateEnd ='1')) then
      s_stateBitGate <= '0';
    elsif (falling_edge(p_gateWindow)) then
      s_stateBitGate <= '1';
    else
      s_stateBitGate <= s_stateBitGate;
    end if;
  end process;

  -- Wait time
  process (p_reset, s_stateBitGate, p_clk500mHz)
  begin
    if ((p_reset = '1') or (s_stateBitGate = '0')) then
      s_cntGateWidth <= "00";
      s_gateEnd      <= '0';
    elsif (rising_edge(p_clk500mHz)) then
      if (s_cntGateWidth = "11") then
        -- Start the gating windows
        s_cntGateWidth <= s_cntGateWidth;
        s_gateEnd      <= '1';
      else
        s_cntGateWidth <= s_cntGateWidth + 1;
        s_gateEnd      <= '0';
      end if;
    end if;
  end process;

   -- Start
  process (p_reset, s_beamLostEnd, p_beamLost)
  begin
    if ((p_reset = '1') or (s_beamLostEnd ='1')) then
      s_stateBitbeamLost <= '0';
    elsif (falling_edge(p_beamLost)) then
      s_stateBitbeamLost <= '1';
    else
      s_stateBitbeamLost <= s_stateBitbeamLost;
    end if;
  end process;

  -- Wait time
  process (p_reset, s_stateBitbeamLost, p_clk500mHz)
  begin
    if ((p_reset = '1') or (s_stateBitbeamLost = '0')) then
      s_cntbeamLostWidth <= "00";
      s_beamLostEnd      <= '0';
    elsif (rising_edge(p_clk500mHz)) then
      if (s_cntbeamLostWidth = "11") then
        -- Start the gating windows
        s_cntbeamLostWidth <= s_cntbeamLostWidth;
        s_beamLostEnd      <= '1';
      else
        s_cntbeamLostWidth <= s_cntbeamLostWidth + 1;
        s_beamLostEnd      <= '0';
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  p_stateBitGate     <= s_stateBitGate;
  p_stateBitbeamLost <= s_stateBitbeamLost;

end architecture rtl_topup_stateBit;

